	.LI OFF
	.CR Z80
	.OR $82F9
	.TF ram-dos.bin,BIN	

HDLEN	 .EQ 18
FCB_ADR  .EQ $82D0		; FCB size is 37 bytes, so 82D0 should be enough to fit
				; between RAMDisk code end (82b5) and our code start (8300)


; MSX-bload format header (bloadable file will be created)
	.DB $FE
	.DW $8300
	.DW END
	.DW START

VRAMPG: .DB 7			; Number of VRAM pages used in RAMDISK (0-7)
RAMPG:  .DB 3			; Number of RAM  pages used in RAMDISK 
				; (1-3 on 128K machine, 1-8 on machine with 256K or more)
USEDOS: .DB 1			; if 1, save msxdos.sys, command.com and k7.com to ramdisk
	
PBYTE:	.DB 0	

START:
	ld   ix,PBYTE			; find RAM and switch it on pages 1,2,3
PLOOP:
	ld   a,(ix+$00)
	ld   hl,$5500
	push hl
	call $0024
	pop  hl
	ld   a,(hl)
	cpl
	ld   (hl),a
	ld   b,(hl)
	inc  (ix+$00)
	cp   b
	jr   nz,PLOOP

	ld     a,4			; DISK.ROM will be stored in RAM page 4
	out    ($FD),a

	ld   hl,PACKED_RAMDISK		; unpack DISK.ROM & RAMDisk code (4000-82B5)
	ld   de,$4000
	call Unpack
	
	ld a,(VRAMPG)			; set the number of VRAM pages used by RAMDisk
	ld ($8125),a

	ld a,(RAMPG)			; set size of RAM part of RAMDisk
	ld ($81CF),a
	ld l,a
	ld h,0
	add hl,hl
	add hl,hl
	add hl,hl
	add hl,hl
	dec hl
	dec hl
	dec hl
	ld ($807e),hl

	in   a,($a8)			; patch RAMDisk with correct value for memory slot
	and  $F0
	ld   b,a
	rrca
	rrca
	rrca
	rrca
	or   b
	ld   ($81A0),a
	ld   ($81A9),a

	ld   a,($FFFF)			; patch RAMDisk with correct value for memory subslot
	cpl
	and  $F0
	ld   b,a
	rrca
	rrca
	rrca
	rrca
	or   b
	ld   ($81AD),a

	ld   hl,PACKED_COMMAND_COM	; unpack COMMAND.COM now, because during RAMDisk initialization
	ld   de,FILE_ADR		; memory at 9000, where packed COMMAND.COM sits, will be erased
	call Unpack

	call $8000			; initialize RAMDisk.

	ld   a,(USEDOS)			; don't copy system files to RAMDisk if USEDOS=0
	or   a
	jr   z,NO_DOS

	ld   hl,COMMAND_COM_NAME	; save COMMAND.COM to RAMDisk
	ld   de,FILE_ADR+$1A00		; 1A00 is the filesize of COMMAND.COM we use
	call DISK_SAVE

	ld   hl,PACKED_MSXDOS_SYS	; unpack MSXDOS.SYS
	ld   de,FILE_ADR
	call Unpack
	ld   hl,MSXDOS_SYS_NAME		; save MSXDOS.SYS to RAMDisk
	call DISK_SAVE

	ld   hl,PACKED_K7_COM		; unpack K7.COM
	ld   de,FILE_ADR
	call Unpack
	ld   hl,K7_COM_NAME		; save K7.COM to RAMDisk
	call DISK_SAVE

	ld hl,$fbf0			; execute K7<enter> command after entering DOS
	ld ($f3fa),hl
	ld (hl),'K'
	inc hl
	ld (hl),'7'
	inc hl
	ld (hl),$0D
	inc hl
	ld ($f3f8),hl
	
NO_DOS:
GO:
	ld   a,1			; GO ! (boot MSX-DOS or go to BASIC)
	ld   ($F347),a
	jp   $5A11

; ------------------------------------	
; ------------------------------------	
; ------------------------------------	
DISK_SAVE:
	push hl
	ld   hl,FILE_ADR
	ex   de,hl
	xor  a
	sbc  hl,de
	ld b,h
	ld c,l
	pop hl
	push bc				; store filesize
	ld   de,FCB_ADR			; prepare FCB
	xor  a
	ld   (de),a
	inc de				; use current drive
	ld   bc,$000B
	ldir				; set filename
	ld   hl,FCB_ADR+$C
	ld   de,FCB_ADR+$D
	ld   (hl),0
	ld   bc,$0018
	ldir
	ld   de,FCB_ADR			; create file
	ld   c,$16
	call $F37D
	ld   hl,1			
	ld   (FCB_ADR+$E),hl		; set block size
	ld   de,FILE_ADR		; set transfer address
	ld   c,$1A
	call $F37D
	pop  hl				; get filesize and write file
	ld   de,FCB_ADR
	ld   c,$26
	call $F37D
	ld   de,FCB_ADR			; close file
	ld   c,$10
	call $F37D
	ret
; ------------------------------------	
; ------------------------------------	
; ------------------------------------	
Unpack:
;	HL=from, DE=to

	di
        LD BC,HDLEN			; Skip the "RNC ..." header
        ADD HL,BC			;
        SCF
        LD A,(HL)
        INC HL
        ADC A,A
        ADD A,A
        JR JR_XLOOP

;---------------------------------------------------------

FETCH3:
        LD A,(HL)
        INC HL
        ADC A,A
        jr BACK3
FETCH4:
        LD A,(HL)
        INC HL
        ADC A,A
        jr BACK4
FETCH5:
        LD A,(HL)
        INC HL
        ADC A,A
        Jr BACK5
FETCH6:
        LD A,(HL)
        INC HL
        ADC A,A
        Jr BACK6
FETCH7:
        LD A,(HL)
        INC HL
        ADC A,A
        jr BACK7
RAW:
        LD C,4
X4BITS:
        ADD A,A
        JR Z,FETCH7
BACK7:
        RL B
        DEC C
        JR NZ,X4BITS

        PUSH AF
        LD A,3
        ADD A,B
        ADD A,A
        LD C,A
RAWLPB:
        LDI
        INC BC

        LDI
        INC BC

        DEC C
        JR NZ,RAWLPB

        POP AF
JR_XLOOP:
        Jr XLOOP

;---------------------------------------------------------

FETCH0:
        LD A,(HL)
        INC HL
        ADC A,A
        JR C,SMALLS
GETLEN:
        ADD A,A
        JR Z,FETCH3
BACK3:
        RL C
        ADD A,A
        JR Z,FETCH4
BACK4:
        JR NC,COPY
        ADD A,A
        JR Z,FETCH5
BACK5:
        DEC C
        PUSH HL

        LD H,A
        LD A,C
        ADC A,A
        LD C,A
        CP 9
        LD A,H

        POP HL
        JR Z,RAW

;---------------------------------------------------------

COPY:
        ADD A,A
        JR Z,FETCH6
BACK6:
        JR NC,BYTEDISP
        ADD A,A
        JR NZ,SKIP0
        LD A,(HL)
        INC HL
        ADC A,A
SKIP0:
        RL B
        ADD A,A
        JR NZ,SKIP1
        LD A,(HL)
        INC HL
        ADC A,A
SKIP1:
        JR C,BIGDISP
        INC B
        DEC B
        JR NZ,BYTEDISP
        INC B
ANOTHER:
        ADD A,A
        JR NZ,DISPX
        LD A,(HL)
        INC HL
        ADC A,A
DISPX:
        RL B
BYTEDISP:
        PUSH AF
        LD A,E
        SUB (HL)
        PUSH HL
        LD L,A
        LD A,D
        SBC A,B
        LD H,A
        DEC HL
BYTELP:
        LDI
        INC BC
        DEC C
        JR NZ,BYTELP

        POP HL
        INC HL

        POP AF
        JR XLOOP

;---------------------------------------------------------

GETBITS:
        LD A,(HL)
        INC HL
        ADC A,A
        JR C,STRING
XBYTE:
        ld b,a
        LDI
        INC BC
        ld a,b
XLOOP:
        ADD A,A
        JR C,CHKZ

        ld b,a
        LDI
        INC BC
        ld a,b

        ADD A,A
        JR NC,XBYTE
CHKZ:
        JR Z,GETBITS

;---------------------------------------------------------

STRING:
        LD BC,2
        ADD A,A
        JR Z,FETCH0
        JR NC,GETLEN
SMALLS:
        ADD A,A
        JR Z,FETCH1
BACK1:
        JR NC,BYTEDISP

        INC C
        ADD A,A
        JR Z,FETCH2
BACK2:
        JR NC,COPY

        LD C,(HL)
        INC HL
        INC C
        DEC C
        JR Z,OVERNOUT

        PUSH AF

        LD A,C
        ADD A,8
        LD C,A

        POP AF
        Jr COPY

;---------------------------------------------------------

BIGDISP:
        ADD A,A
        JR NZ,SKIP2
        LD A,(HL)
        INC HL
        ADC A,A
SKIP2:
        RL B
        SET 2,B
        ADD A,A
        JR NZ,SKIP3
        LD A,(HL)
        INC HL
        ADC A,A
SKIP3:
        JR C,BYTEDISP
        Jr ANOTHER
FETCH1:
        LD A,(HL)
        INC HL
        ADC A,A
        jr BACK1
FETCH2:
        LD A,(HL)
        INC HL
        ADC A,A
        jr BACK2
OVERNOUT:
        ADD A,A
        JR NZ,CHECK4END

        LD A,(HL)
        INC HL
        ADC A,A
CHECK4END:
        JR C,XLOOP

	RET
; ------------------------------------	
; ------------------------------------	
; ------------------------------------	

MSXDOS_SYS_NAME:
	.AS "MSXDOS  SYS"
PACKED_MSXDOS_SYS:
	.BI msxdos.pak

COMMAND_COM_NAME:
	.AS "COMMAND COM"
PACKED_COMMAND_COM:
	.BI command.pak

K7_COM_NAME:
	.AS "K7      COM"
PACKED_K7_COM:
	.BI k7.pak

PACKED_RAMDISK:
FILE_ADR:
	.BI ramdisk.pak

END:
