	.LI OFF
;---------------------------------------------------------
; PRO-PACK Unpack Source Code - Z80, Method 2
;
; Copyright [c) 1992 Rob Northen Computing
;
; File: RNC_2.ASM
;
; Date: 23.03.92
;
; V1.0  - Original
; V1.1  - Optimized by Jeff Frohwein, 99-Jul-25
;         Unpacking is now ~7% faster.
; V1.1M - Adapted for standard (non-gameboy) Z80 by Stanislav Borutsky, June 2003
;         Tested on MSX machine. Assembled with SBASM - http://www.xs4all.nl/~sbp/sbasm/
;---------------------------------------------------------
; On entry,
;   HL points to PACKED data in ROM
;   DE points to RAM area for the UNPACKED data
; On exit,
;   all general purpose registers are undefined - A,B,C,D,E,H,L.
;---------------------------------------------------------

	.CR Z80
HDLEN	.EQ 18
	.OR $E000
	.TF unpack.bin
	
START:	JR PRE_UNPACK
	
FILE_TYPE:
	.DB 0				; FF (BASIC) or 41 (ROM) or 00 (COM) will be set here
PACKED_ADDRESS:
	.DW 0				; Packed data address will be set
UNPACK_ADDRESS:
	.DW $4000			; Destination address will be set
FILE_SIZE:
	.DW 0				; 0 if packed, real size if not packed will be set
PBYTE:
        .DB 0
PRE_UNPACK:
	ld   de,($F676)			; basic program address (TXTTAB)
	dec  de
	ld   a,(FILE_TYPE)
	inc  a
	jr   z,U_BAS			; don't switch memory for basic file loading
	
        ld   ix,PBYTE
PLOOP:
        ld   a,(ix+$00)
        ld   hl,$5500
        push hl
        call $0024
        pop  hl
        ld   a,(hl)
        cpl
        ld   (hl),a
        ld   b,(hl)
        inc  (ix+$00)
        cp   b
        jr   nz,PLOOP

	ld   a,(FILE_TYPE)
	or   a
	jr   nz,U_BIN
	
	in   a,($a8)
	and  $FC
	ld   b,a
	rlca
	rlca
	and  $03
	or   b
	out  ($a8),a

	ld   a,($FFFF)
	cpl
	and  $FC
	ld   b,a
	rlca
	rlca
	and  $03
	or   b
	ld   ($FFFF),a
	
	ld   a,$C9
	ld   (0005),a
U_BIN:	
	LD DE,(UNPACK_ADDRESS)				
U_BAS:
	LD HL,(PACKED_ADDRESS)

	LD BC,(FILE_SIZE)
	LD A,B
	OR C
	JR Z,Unpack
	LDIR
	JP AFTER_PACK

Unpack:
	di

        LD BC,HDLEN			; Skip the "RNC ..." header
        ADD HL,BC			;
        SCF
        LD A,(HL)
        INC HL
        ADC A,A
        ADD A,A
        JR JR_XLOOP

;---------------------------------------------------------

FETCH3:
        LD A,(HL)
        INC HL
        ADC A,A
        jr BACK3
FETCH4:
        LD A,(HL)
        INC HL
        ADC A,A
        jr BACK4
FETCH5:
        LD A,(HL)
        INC HL
        ADC A,A
        Jr BACK5
FETCH6:
        LD A,(HL)
        INC HL
        ADC A,A
        Jr BACK6
FETCH7:
        LD A,(HL)
        INC HL
        ADC A,A
        jr BACK7
RAW:
        LD C,4
X4BITS:
        ADD A,A
        JR Z,FETCH7
BACK7:
        RL B
        DEC C
        JR NZ,X4BITS

        PUSH AF
        LD A,3
        ADD A,B
        ADD A,A
        LD C,A
RAWLPB:
        LDI
        INC BC

        LDI
        INC BC

        DEC C
        JR NZ,RAWLPB

        POP AF
JR_XLOOP:
        Jr XLOOP

;---------------------------------------------------------

FETCH0:
        LD A,(HL)
        INC HL
        ADC A,A
        JR C,SMALLS
GETLEN:
        ADD A,A
        JR Z,FETCH3
BACK3:
        RL C
        ADD A,A
        JR Z,FETCH4
BACK4:
        JR NC,COPY
        ADD A,A
        JR Z,FETCH5
BACK5:
        DEC C
        PUSH HL

        LD H,A
        LD A,C
        ADC A,A
        LD C,A
        CP 9
        LD A,H

        POP HL
        JR Z,RAW

;---------------------------------------------------------

COPY:
        ADD A,A
        JR Z,FETCH6
BACK6:
        JR NC,BYTEDISP
        ADD A,A
        JR NZ,SKIP0
        LD A,(HL)
        INC HL
        ADC A,A
SKIP0:
        RL B
        ADD A,A
        JR NZ,SKIP1
        LD A,(HL)
        INC HL
        ADC A,A
SKIP1:
        JR C,BIGDISP
        INC B
        DEC B
        JR NZ,BYTEDISP
        INC B
ANOTHER:
        ADD A,A
        JR NZ,DISPX
        LD A,(HL)
        INC HL
        ADC A,A
DISPX:
        RL B
BYTEDISP:
        PUSH AF
        LD A,E
        SUB (HL)
        PUSH HL
        LD L,A
        LD A,D
        SBC A,B
        LD H,A
        DEC HL
BYTELP:
        LDI
        INC BC
        DEC C
        JR NZ,BYTELP

        POP HL
        INC HL

        POP AF
        JR XLOOP

;---------------------------------------------------------

GETBITS:
        LD A,(HL)
        INC HL
        ADC A,A
        JR C,STRING
XBYTE:
        ld b,a          ; Took out 'push af' - JF, 99-Jul-25
        LDI
        INC BC
        ld a,b          ; Took out 'pop af' - JF, 99-Jul-25
XLOOP:
        ADD A,A
        JR C,CHKZ

        ld b,a          ; Took out 'push af' - JF, 99-Jul-25
        LDI
        INC BC
        ld a,b          ; Took out 'pop af' - JF, 99-Jul-25

        ADD A,A
        JR NC,XBYTE
CHKZ:
        JR Z,GETBITS

;---------------------------------------------------------

STRING:
        LD BC,2
        ADD A,A
        JR Z,FETCH0
        JR NC,GETLEN
SMALLS:
        ADD A,A
        JR Z,FETCH1
BACK1:
        JR NC,BYTEDISP

        INC C
        ADD A,A
        JR Z,FETCH2
BACK2:
        JR NC,COPY

        LD C,(HL)
        INC HL
        INC C
        DEC C
        JR Z,OVERNOUT

        PUSH AF

        LD A,C
        ADD A,8
        LD C,A

        POP AF
        Jr COPY

;---------------------------------------------------------

BIGDISP:
        ADD A,A
        JR NZ,SKIP2
        LD A,(HL)
        INC HL
        ADC A,A
SKIP2:
        RL B
        SET 2,B
        ADD A,A
        JR NZ,SKIP3
        LD A,(HL)
        INC HL
        ADC A,A
SKIP3:
        JR C,BYTEDISP
        Jr ANOTHER
FETCH1:
        LD A,(HL)
        INC HL
        ADC A,A
        jr BACK1
FETCH2:
        LD A,(HL)
        INC HL
        ADC A,A
        jr BACK2
OVERNOUT:
        ADD A,A
        JR NZ,CHECK4END

        LD A,(HL)
        INC HL
        ADC A,A
CHECK4END:
        JR C,XLOOP

; -----------------------
; run BASIC code:
AFTER_PACK:	
	ld   a,(FILE_TYPE)
	or   a
	JP   Z,$0100
	inc  a
	jr   nz,EXEC_ROM

	INC DE
	LD HL,$F6C2
	LD (HL),E
	INC HL
	LD (HL),D

; set BASIC memory config
	in  a,($a8)
	and $F0
	out ($a8),a
	
	JP $73AC

; run the ROM:
EXEC_ROM:
	LD HL,(UNPACK_ADDRESS)
	INC HL
	INC HL
	LD E,(HL)
	INC HL
	LD D,(HL)
	EX DE,HL
	JP (HL)
END:
